
//Title:        MiniTeller
//Version:      
//Copyright:    Copyright (c) 1998
//Author:       Nicholas Riley
//Company:      Brandeis University
//Description:  A small bank implementation

package miniteller;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import borland.jbcl.layout.*;

class AcctListFrame extends JFrame implements ListSelectionListener {
  BorderLayout panelsLayout = new BorderLayout();
  GridBagLayout detailGBL = new GridBagLayout();
  JScrollPane scrollPane;
  JPanel detailPanel = new JPanel();
  JLabel lnLabel = new JLabel("Customer last name:", SwingConstants.RIGHT);
  JTextField lnField = new JTextField();
  JLabel fnLabel = new JLabel("First name:", SwingConstants.RIGHT);
  JTextField fnField = new JTextField();
  JLabel addrLabel = new JLabel("Address:", SwingConstants.RIGHT);
  JTextField addr1Field = new JTextField();
  JTextField addr2Field = new JTextField();
  JTextField phoneField = new JTextField();
  JLabel phoneLabel = new JLabel("Phone:", SwingConstants.RIGHT);
  JTextField ssnField = new JTextField();
  JLabel ssnLabel = new JLabel("SSN:", SwingConstants.RIGHT);
  JLabel explLabel = new JLabel(
    "Double-click a row in the table below to see detailed information:",
    SwingConstants.CENTER);
  JTable table;

  protected MiniTeller mt = null;

  AcctTableModel tmodel;
  Account[] accts = null;

  protected AcctListFrame() throws AccessViolation {
    this(null);
  }

  public AcctListFrame(MiniTeller teller) throws AccessViolation {
    this.mt = teller;
    tmodel = new AcctTableModel();
    jbInit();
    pack();
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
  }

  private void jbInit() {

    this.setTitle("Account List");
    this.getContentPane().setLayout(panelsLayout);

    fnField.setColumns(15);
    fnField.setBackground(SystemColor.control);
    fnField.setEditable(false);
    lnField.setColumns(15);
    lnField.setBackground(SystemColor.control);
    lnField.setEditable(false);
    addr1Field.setColumns(30);
    addr1Field.setBackground(SystemColor.control);
    addr1Field.setEditable(false);
    panelsLayout.setVgap(10);
    addr2Field.setColumns(30);
    addr2Field.setBackground(SystemColor.control);
    addr2Field.setEditable(false);
    phoneField.setColumns(15);
    phoneField.setBackground(SystemColor.control);
    phoneField.setEditable(false);
    ssnField.setColumns(11);
    ssnField.setBackground(SystemColor.control);
    ssnField.setEditable(false);
    addrLabel.setAlignmentY((float) 0.0);

    ListSelectionModel selmodel = new DefaultListSelectionModel();
    table = new JTable(tmodel, null, selmodel);
    initColumnSizes(table, tmodel);
    selmodel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    selmodel.addListSelectionListener(this);
    table.setPreferredScrollableViewportSize(new Dimension(500, 70));

    scrollPane = new JScrollPane(table);

    detailPanel.setLayout(detailGBL);
    detailPanel.add(lnLabel, new GridBagConstraints2(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 10, 5, 0), 0, 0));
    detailPanel.add(lnField, new GridBagConstraints2(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 10), 0, 0));
    detailPanel.add(fnLabel, new GridBagConstraints2(3, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
    detailPanel.add(fnField, new GridBagConstraints2(4, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 10), 0, 0));
    detailPanel.add(addrLabel, new GridBagConstraints2(1, 2, 1, 2, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.BOTH, new Insets(0, 0, 5, 0), 0, 0));
    detailPanel.add(addr1Field, new GridBagConstraints2(2, 2, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 2, 10), 0, 0));
    detailPanel.add(addr2Field, new GridBagConstraints2(2, 3, 4, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 10), 0, 0));
    detailPanel.add(phoneLabel, new GridBagConstraints2(1, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
    detailPanel.add(phoneField, new GridBagConstraints2(2, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.SOUTHEAST, GridBagConstraints.BOTH, new Insets(0, 5, 5, 10), 0, 0));
    detailPanel.add(ssnLabel, new GridBagConstraints2(3, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 5, 0), 0, 0));
    detailPanel.add(ssnField, new GridBagConstraints2(4, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 5, 10), 0, 0));
    this.getContentPane().add(explLabel, BorderLayout.NORTH);
    this.getContentPane().add(scrollPane, BorderLayout.CENTER);
    this.getContentPane().add(detailPanel, BorderLayout.SOUTH);

  }

  private void initColumnSizes(JTable table, AcctTableModel model) {
    TableColumn column = null;
    Component comp = null;
    int headerWidth = 0;
    int cellWidth = 0;
    Object[] longValues = model.longValues;

    for (int i = 0; i < COLCOUNT; i++) {
      column = table.getColumnModel().getColumn(i);

      comp = column.getHeaderRenderer().
                       getTableCellRendererComponent(
                           null, column.getHeaderValue(),
                           false, false, 0, 0);
      headerWidth = comp.getPreferredSize().width;

      if (i == COLINDEX_ACCTNO || i == COLINDEX_BAL) {
        DefaultTableCellRenderer cr = new DefaultTableCellRenderer();
        cr.setHorizontalAlignment(SwingConstants.RIGHT);
        column.setCellRenderer(cr);
      }

      comp = table.getDefaultRenderer(model.getColumnClass(i)).
                       getTableCellRendererComponent(
                           table, longValues[i],
                           false, false, 0, i);
      cellWidth = comp.getPreferredSize().width;
      column.setMinWidth(Math.max(headerWidth, cellWidth));
    }
  }

  // column indices; count of columns in table
  static final int COLINDEX_ACCTNO = 0, COLINDEX_NAME = 1, COLINDEX_BAL = 2;
  static final int COLCOUNT = 3;

  class AcctTableModel extends AbstractTableModel {

    final String[] columnNames = {"Account #",
                            "Customer Name",
                            "Current Balance"};
    public final Object[] longValues = {
      new Integer(999999), "MMMMMMMMMMMMMMM, MMMMMMMMMMMMMMM",
      "$999999999.99"};
    String[][] data; // table data
    int numaccts;

    public AcctTableModel() throws AccessViolation {
      super();
      // populate table with account basic information from MiniTeller
      accts = mt.getAccounts();
      numaccts = accts.length;
      if (numaccts == 0) {
        explLabel.setText("No accounts have been created.");
        return;
      }

      String[] binfo;
      data = new String[numaccts][COLCOUNT];
      for (int i = 0 ; i < numaccts ; i++) {
        binfo = accts[i].getBasicInfo();
        data[i][COLINDEX_ACCTNO] = binfo[Account.bidx_acct];
        data[i][COLINDEX_NAME] = binfo[Account.bidx_name];
        data[i][COLINDEX_BAL] = binfo[Account.bidx_bal];
      }
    }

    public int getColumnCount() {
      return columnNames.length;
    }

    public int getRowCount() {
      return (data == null ? 0 : data.length);
    }

    public boolean isCellEditable(int row, int col) {
      return false;
    }

    public String getColumnName(int col) {
        return columnNames[col];
    }

    public Object getValueAt(int row, int col) {
        if (numaccts == 0) return "";
        return data[row][col];
    }

    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }
  }

  protected void fatal(org.omg.CORBA.UserException e) {
    new ErrorDialog(this, e);
    dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
  }

  int currIndex = -1;

  public void valueChanged(ListSelectionEvent e) {
    // Attempt to intercept a second click on a row by noting
    // that the first and last indices will be the same. This
    // works in all circumstances but the first click, in which
    // case currIndex = -1 is changed to currIndex = -2, to
    // provide an ignore signal.
    // Calling getSelectedRow() does a similar thing, but it
    // doesn't handle double-clicking this way.
    int idx = e.getFirstIndex(); int lidx = e.getLastIndex();
    if (currIndex == -1) {
      currIndex = -2;
      return;
    }
    if (idx != lidx || idx == currIndex) return;
    String[] cinfo;
    currIndex = idx;
    try {
    cinfo = accts[idx].getCustomerInfo();
    } catch (AccessViolation ae) {
      ae.explanation = ae.explanation + ". Aborting!";
      fatal(ae); return;
    }
    fnField.setText(cinfo[Account.cidx_first]);
    lnField.setText(cinfo[Account.cidx_last]);
    addr1Field.setText(cinfo[Account.cidx_addr1]);
    addr2Field.setText(cinfo[Account.cidx_addr2]);
    phoneField.setText(cinfo[Account.cidx_phone]);
    ssnField.setText(cinfo[Account.cidx_ssn]);
  }

}
