
//Title:        MiniTeller
//Version:      
//Copyright:    Copyright (c) 1998
//Author:       Nicholas Riley
//Company:      Brandeis University
//Description:  A small bank implementation

package miniteller;

import java.awt.*;
import com.sun.java.swing.*;
import java.awt.event.*;

class AcctOpsFrame extends JFrame implements ActionListener {
  JPanel acctPanel = new JPanel();
  JLabel acctLabel = new JLabel("Please enter an account number:");
  JTextField acctField = new JTextField();
  JLabel middleLabel = new JLabel("Then click Continue.");
  JButton contButton = new JButton("Continue");
  JTabbedPane opsPane = new JTabbedPane();
  CardLayout stageLayout = new CardLayout();
  JPanel stagePanel = new JPanel(stageLayout);

  public static final int OP_DEPOSIT = 0, OP_WITHDRAW = 1, OP_TRANSFER = 2;
  // first and last indices for iterating through OP_ constants above
  protected static final int OP_FINDEX = 0, OP_LINDEX = 2;
  protected static final String[] OP_ACTION = {"Deposit", "Withdraw", "Transfer"},
                                  OP_PREP = {"into", "from", "from"};
  protected int op;

  MiniTeller mt = null;
  Account acct = null;
  Float bal;

  public AcctOpsFrame(MiniTeller teller, int opCode) {
    this.mt = teller;
    this.op = opCode;
    jbInit();
    pack();
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
  }

  private void jbInit() {
    this.setTitle("Account Operations");
    acctField.setColumns(5);
    acctField.setHorizontalAlignment(JTextField.RIGHT);
    stagePanel.add(contButton, "continue");
    contButton.addActionListener(this);
    this.getContentPane().add(acctPanel, BorderLayout.NORTH);
    acctPanel.add(acctLabel, null);
    acctPanel.add(acctField, null);
    this.getContentPane().add(middleLabel, BorderLayout.CENTER);
    for (int oi = OP_FINDEX ; oi <= OP_LINDEX ; oi++)
      opsPane.addTab(OP_ACTION[oi], new OpPanel(oi, OP_ACTION[oi], OP_PREP[oi],
                                                this));
    opsPane.setSelectedIndex(op);
    stagePanel.add(opsPane, "ops");
    this.getContentPane().add(stagePanel, BorderLayout.SOUTH);
  }

  public void actionPerformed(ActionEvent e) {
    try {
    acct = mt.getAccount(Integer.parseInt(acctField.getText()));
    bal = new Float(acct.getBalance());
    } catch (AccessViolation ae) {
      new ErrorDialog(this, ae);
      return;
    } catch (NumberFormatException fe) {
      new ErrorDialog(this, new DataInvalid(
        "Account number in wrong format - please try again."));
      return;
    }
    acctField.setEditable(false);
    acctField.setBackground(SystemColor.control);
    stageLayout.next(stagePanel); // notifies OpPanels that we're ready
    repaint(); // avoids extraneous but rather neat-looking button hilite
    middleLabel.setText("Select an operation:");
  }
}
